/*-------------------------------------------------------------------------*\

   FuncExpr.h

   Func expression - a function call expression
   
   Copyright (C) 1996 by Michael A. Muller
      
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
      
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.      

\*-------------------------------------------------------------------------*/

#file FuncExpr

#intinclude Expr
#intinclude Ptrs
#intinclude PtrSList

#impinclude VarInst
#impinclude FuncVar
#impinclude FuncVal

class Context;

class FuncExpr : public Expr {
   private:
      Context *context;
      ObjPtr<Expr> func;
      PtrSList<Expr> parms;
   public:
      FuncExpr(Context *context0, Expr *expr0, PtrSList<Expr> &parms0) :
               context(context0),
               func(expr0),
               parms(parms0)
         {
         if (expr0->classId() != ClassIds::VarInst ||
             ((VarInst*)expr0)->getVar()->classId() != ClassIds::FuncVar)
            throw 0;
         }
         
      virtual ClassId classId() const
         {
         return ClassIds::FuncExpr;
         }
         
      bool isStringExpr() const
         {
         // XXX this needs to be assessed at runtime. XXX
         return true;
         }
         
      virtual Value *evaluate() const
         {
         PtrSList<Value> parmVals;
         for (Pix p = parms.first(); p; parms.next(p))
            parmVals.append(parms(p)->evaluate());
         ObjPtr<FuncVal> funcVal((FuncVal*)((VarInst*)(Expr*)func)->evaluate());
         return funcVal->evaluate(context, parmVals);
         }
      
   }




