/*-------------------------------------------------------------------------*\

   FuncVar.h

   Function Variable - a Variable which represents a function.
   
   Copyright (C) 1996, Michael A. Muller

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
      
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.      

\*-------------------------------------------------------------------------*/
#file FuncVar

#intinclude Ptrs
#intinclude Var
#intinclude Value

#impinclude PFuncVal

class FuncVal;

class FuncVar : public Var {
   private:
      ObjPtr<Value> _val;

   public:
      FuncVar(const char *name, const FuncVal &val0) :
		      Var(name),
		      _val(val0.classId() == ClassIds::PFuncVal ? 
		               new PFuncVal((const PFuncVal &)val0) : 0
		           )
		   {
		   if (!_val)
		      throw "Attempted to construct a FuncVar with a non-PFuncVal!!!";
		   }

      FuncVar(const FuncVar &other) :
		      Var(other.name()),
		      _val(other.val().classId() == ClassIds::PFuncVal ? 
		           new PFuncVal((const PFuncVal &)other.val()) : 0
		           )
		   {
		   }
      ~FuncVar()
			{
			}
      
      const FuncVal &val() const
         {
         return *(FuncVal*)(Value*)_val;
         }
         
      FuncVar &val(const FuncVal &newVal)
		   {
		   *_val = newVal;
		   return *this;
		   }
      
      virtual ClassId classId() const { return ClassIds::FuncVar; }
      
      virtual Value *evaluate() const
		   {
		   return _val->classId() == ClassIds::PFuncVal ?
		      new PFuncVal(*(PFuncVal*)(Value*)_val) :
		      0;
		   }
      
      virtual bool isStringExpr() const
         {
         return false;
         }
      
      virtual FuncVar *copy() const
         {
         return new FuncVar(*this);
         }
      }


