#file IndxExpr

#intinclude Expr

class Context;

/*---------------------------------------------------------------------------*\
	An Index Expression is an operation that can be performed on a table.
	Given a table "t", the expression "t[i]" evaluates to the value of the
	first column of the table at row index "i".  The expression "t[i;e]"
	returns the value of the expression e when evaluated in the context of
	the row at index "i".  "i" is assumed to be zero based.
\*---------------------------------------------------------------------------*/
class IndxExpr : public Expr {
	private:
		Context *context;
		ObjPtr<Expr> table;
		ObjPtr<Expr> index;
		ObjPtr<Expr> toExpand;
	
	public:
		/*---------------------------------------------------------------------*\
			Creates an index expression given the context, table, index, and
			an expression to evaluate.
		\*---------------------------------------------------------------------*/
		IndxExpr(Context *context0, Expr *table0, Expr *index0,
					Expr *toExpand0) :
				context(context0),
				table(table0),
				index(index0),
				toExpand(toExpand0)
			{
         if (table0->classId() != ClassIds::VarInst ||
             ((VarInst*)table0)->getVar()->classId() != ClassIds::TableVar)
            throw 0;
			if (!index0->classId()->isStringExpr())
				throw 0;
			}
		
		/*---------------------------------------------------------------------*\
			Returns the expressions class id.
		\*---------------------------------------------------------------------*/
      virtual ClassId classId() const
         {
         return ClassIds::IndxExpr;
         }
         
      bool isStringExpr() const
         {
         // XXX this needs to be assessed at runtime. XXX
         return true;
         }
         
		/*---------------------------------------------------------------------*\
			Evaluates the index expression by locating the index and 
			evaluating the expression.
		\*---------------------------------------------------------------------*/
      virtual Value *evaluate() const
         {
			// first of all, get the table
			ObjPtr<TableVal> tab = (TableVal*)table->evaluate();
			
			ObjPtr<StrVal> indx = (StrVal*)index->evaluate();
			
			int actualIndex = atoi(*indx);
			
         for (Pix p = tab->first(); p && actualIndex--; tab->next(p));
			
			if (!p)
				return new StrVal("");
		
			// if there is no "to expand" expression, just return the 0th column
			if (!toExpand)
				return new StrVal((*tab)(p)[0]);
			
         ObjPtr<FuncVal> funcVal((FuncVal*)((VarInst*)(Expr*)func)->evaluate());
         return funcVal->evaluate(context, parmVals);
         }

	};