/*-------------------------------------------------------------------------*\

   PFuncVal.d

   Primitive Function Value - holds the address of a primitive function.
   
   Copyright (C) 1996, Michael A. Muller

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
      
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.      

\*-------------------------------------------------------------------------*/

#file PFuncVal

#intinclude String
#intinclude Value
#intinclude FuncVal

class PFuncVal : public FuncVal {
   public:
      typedef Value *(*PrimitiveFunction)(Context *context,
                                          const PtrSList<Value> &parms);

   private:
		String funcName;
      PrimitiveFunction primVal;
		int parmCount;
      
   public:
      PFuncVal(const char *funcName0, PrimitiveFunction primVal0, int parmCount0) :
				funcName(funcName0),
				primVal(primVal0),
				parmCount(parmCount0)
         {
         }
         
      virtual ClassId classId() const
			{
			return ClassIds::PFuncVal;
			}
      
      virtual Value *evaluate(Context *context,
                              const PtrSList<Value> &parms) const
         {
			if (parms.length() != parmCount)
				{
				cerr << "Bad parm list for function " << funcName
					<< ", " << parmCount << " parameters required." << endl;
				throw 1;
				}

         return primVal(context, parms);
         }
   }



