/*-------------------------------------------------------------------------*\

   Row.h

   A Row is a fixed array of strings - a Row in a grunt table.  Row's can
   be indexed by numerical (0 based) values as well as Pix'es.

   Copyright (C) 1996 by Michael A. Muller
      
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
      
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.      

\*-------------------------------------------------------------------------*/

#file Row

#intinclude Ptrs
#intinclude String
#intinclude Pix

class Row {
   private:
      unsigned numColumns;
      ArrPtr<String> columns;

      // --- had to break out these functions to deal with compiler bug ---

      void copyArray(String *source)
         {
         for (int i = 0; i < numColumns; i++)
            columns[i] = source[i];
         }
         
      void newArray()
         {
         columns = new String[numColumns];
         }
      
   public:
      Row(unsigned numColumns0) :
            numColumns(numColumns0),
            columns(new String[numColumns])
         {
         }
         
      Row() : numColumns(0)
         {
         }
         
      Row(const Row &other) :
            numColumns(other.numColumns),
            columns(numColumns ? new String[numColumns] : 0)
         {
         for (int i = 0; i < numColumns; i++)
            columns[i] = other.columns[i];
         }
         
      Row &operator =(const Row &other)
         {
         if (numColumns && numColumns != other.numColumns)
            throw "Attempted to assign between different sized rows.";
         
         if (other.numColumns)
            {
            numColumns = other.numColumns;
            columns = new String[numColumns];
            }
            
         if (numColumns)
            copyArray(other.columns);
               
         return *this;
         }
         
      Row &init(unsigned numColumns0)
         {
         if (!numColumns)
            {
            numColumns = numColumns0;
            newArray();
            }
         else
            throw "Attempted to 'init' a fully initialized Row.";
            
         return *this;
         }
         
      String &operator [](unsigned index)
         {
         if (index >= numColumns)
            throw "Attempted to reference an out-of-bounds column";
         return columns[index];
         }
         
      const String &operator [](unsigned index) const
         {
         if (index >= numColumns)
            throw "Attempted to reference an out-of-bounds column";
         return columns[index];
         }
         
      String &operator [](Pix p)
         {
         return operator[]((unsigned)p - 1);
         }
         
      const String &operator [](Pix p) const
         {
         return operator[]((unsigned)p - 1);
         }
         
      String &operator ()(Pix p)
         {
         return operator[]((unsigned)p - 1);
         }
         
      const String &operator ()(Pix p) const
         {
         return operator[]((unsigned)p - 1);
         }
      
      Pix first() const
         {
         return (Pix)1;
         }
         
      Pix last() const
         {
         return (Pix)numColumns;
         }
         
      Pix next(Pix &p) const
         {
         if ((unsigned)p < numColumns)
            return (Pix)++((unsigned&)p);
         else
            return (Pix)(p = 0);
         }
         
      Pix prev(Pix &p) const
         {
         if (p)
            return (Pix)--((unsigned&)p);
         else
            return p;
         }
   }
