/*-------------------------------------------------------------------------*\

   Tok.d

   Tokenizer - basic classes needed for a tokenizer for C-like languages.

   Copyright (C) 1996, Michael A. Muller

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
      
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.      

\*-------------------------------------------------------------------------*/
#file Tok

#intinclude String

class ostream;

// imp reqs

#impinclude ctype
#impinclude strstream
#impinclude iostream
#impinclude GrowBuf

#define MAX_EXP 38
#define MIN_EXP -38

struct Token {
   enum { symbol, identifier, integer, bigInteger, floatingPoint, string, end}
      type;
   union {
      char symVal;
      char *identVal;
      int intVal;
      //BigInt bigIntVal;
      float floatVal;
      char *stringVal;
      }

   friend ostream &operator <<(ostream &out, const Token &tok)
      {
      switch (tok.type) {
         case Token::symbol:
            out << '\'' << tok.symVal << '\'';
            break;
         case Token::identifier:
            out << '"' << tok.identVal << '"';
            break;
         case Token::integer:
            out << '\'' << tok.intVal << '\'';
            break;
         case Token::floatingPoint:
            out << '\'' << tok.floatVal << '\'';
            break;
         case Token::string:
            out << '"' << tok.stringVal << '"';
            break;
         case Token::end:
            out << "<End Of Stream>";
            break;
         default:
            out << "---Tokenizer Fucked up!---";
            break;
         }

      return out;
      }


   String asString() const
      {
      ostrstream str;
      switch (type) {
         case symbol:
            str << symVal;
            break;
         case Token::identifier:
            str << identVal;
            break;
         case Token::integer:
            str << intVal;
            break;
         case Token::floatingPoint:
            str << floatVal;
            break;
         case Token::string:
            {
            str << '"';
            for (char *c = stringVal; *c; c++)
               {
               switch (*c)
                  {
                  case '"':
                     str << "\\\"";
                     break;
                  case '\\':
                     str << "\\\\";
                     break;
                  default:
                     str << *c;
                  }
               }
            str << '"';
            break;
            }
         case Token::end:
            str << "<End Of Stream>";
            break;
         default:
            str << "---Tokenizer Fucked up!---";
            break;
         }

      str << '\0';
      return str.str();
      }

   }

class Tokenizer {

   protected:
      const char *sourceName;
      istream &source;
      int lineNum;

      void putBack(char &c)
         {
         if (c == 0)
            return;
         
         if (c == '\n')
            lineNum--;
         source.putback(c);
         c = 0;
         }
      
      char getChar()
         {         
         char c;
         source >> c;
         if (!source)
            return 0;
         if (c == '\n')
            lineNum++;
         return c;
         }         

   public:
      Tokenizer(istream &source, const char *sourceName, int lineNum0 = 1) :
            sourceName(sourceName),
            lineNum(lineNum0),
            source(source)
         {
         }

      Token nextToken()
         {
         short state = 0;
         Token tok;
         char cur, stringTerminator, stringEscChar, *valueToChange;
         int sign, exp, tempExp, expSign;
         char bufferSpace[81];
         GrowableBuffer buf(bufferSpace, 81);

         while (state >= 0) {
            cur = getChar();
 
            switch (state) {
               case 0:
                  // char val == 0 indicates end of file.
                  if (!cur) {
                     state = -1;
                     tok.type = Token::end;
                     }
                  
                  // white space is ignored
                  else if (isspace(cur))
                     ;
                  // alphabetic or underscore signals start of keyword or ident
                  else if (isalpha(cur) || (cur == '_')) {
                     tok.type = Token::identifier;
                     buf.addChar(cur);
                     state = 1;
                     }
               
                  // value is a digit, assume integer
                  else if (isdigit(cur)) {
                     tok.intVal = cur - '0';
                     tok.type = Token::integer;
                     sign = 1;
                     state = 2;
                     }
            
                  // character is a forward slash, could be a comment or a symbol
                  else if (cur == '/')
                     state = 3;
               
                  // character is a double-quote, process double-quoted string
                  else if (cur == '\"') {
                     tok.type = Token::string;
                     stringTerminator = '\"';
                     stringEscChar = '\\';
                     state = 6;
                     }
               
                  // character is a single quote, process singel-quoted string
                  else if (cur == '\'') {
                     tok.type = Token::string;
                     stringTerminator = '\'';
                     stringEscChar = '~';
                     state = 6;
                     }
            
                  // character is a minus sign, could be the beginning of a number
                  // or a symbol
                  else if (cur == '-') {
                     state = 9;
                     tok.symVal = '-';
                     tok.type = Token::symbol;
                     }
                  else {
                     tok.type = Token::symbol;
                     tok.symVal = cur;
                     state = -1;
                     }
                  break;

               // parse identifiers

               case 1:
                  if (isalnum(cur) || cur == '_') {
                     buf.addChar(cur);
                     }
                  else {
                     buf.addChar(0);
                     tok.identVal = buf.getData(); // *** symtabFindOrAdd(bufferGetPtr()); ***
                     state = -1;
                     putBack(cur);
                     }
                  break;

               // parse numbers

               case 2:
                  if (isdigit(cur)) {
                     tok.intVal = tok.intVal * 10 + cur - '0';
                     }
                  else if (cur == '.') {
                     tok.type = Token::floatingPoint;
                     exp = 0;
                     tok.floatVal = tok.intVal;
                     state = 10;
                     }
                  else {
                     tok.intVal *= sign;
                     state = -1;
                     putBack(cur);
                     }
                  break;

               // parse comments

               case 3:
                  if (cur == '*') {
                     state = 4;
                     }
                  else if (cur == '/') {
                     /* ANSI style comment */
                     state = 8;
                     }
                  else {
                     tok.type = Token::symbol;
                     tok.symVal = '/';
                     state = -1;
                     putBack(cur);
                     }
                  break;

               case 4:
                  if (!cur) {
                     /* premature end of file */
                     state = -1;
                     tok.type = Token::end;
                     }
                  else if (cur == '*')
                     state = 5;
                  break;

               case 5:
                  if (!cur) {
                     /* premature end of file */
                     state = -1;
                     tok.type = Token::end;
                     }
                  else if (cur == '/') {
                     state = 0;
                     }
                  else
                     state = 4;
                  break;

               case 8:                       /* process ANSI style // comments */
                  if (!cur) {
                     /* premature end of file */
                     state = -1;
                     tok.type = Token::end;
                     }
                  else if (cur == '\n') {
                     /* end of comment */
                     state = 0;
                     }
                  break;

               case 6:                       /* process string */
                  if (!cur) {
                     buf.addChar(0);
                     tok.identVal = buf.getData();
                     state = -1;
                     }
                  else if (cur == stringTerminator) {
                     buf.addChar(0);
                     tok.stringVal = buf.getData();
                     state = -1;
                     }
                  else if (cur == stringEscChar) {
                     state = 7;
                  }
               else {
                  buf.addChar(cur);
                  }
               break;
   
            case 7:                       /* process string escape sequence */
               if (!cur) {
                  buf.addChar(0);
                  tok.identVal = buf.getData();
                  state = -1;
                  }
               else {
                  switch (cur) {
                     case 't':
                        buf.addChar('\t');
                        break;
                     case 'n':
                        buf.addChar('\n');
                        break;
                     case 'b':
                        buf.addChar('\b');
                        break;
                     case 'r':
                        buf.addChar('\r');
                        break;
                     case '!':
                        valueToChange = &stringEscChar;
                        state = 13;
                        break;
                     case '?':
                        valueToChange = &stringTerminator;
                        state = 13;
                        break;
                     default:
                        buf.addChar(cur);
                     }
    
                  // if we haven't changed the state, chage it to the normal str
                  // state
                  if (state == 7)
                     state = 6;
                  }
               break;
    
            case 13:                   // special escape sequences (\!x and \?x)
                                       // which allow you to change the escape
                                       // char and terminating char.
               if (!cur) {
                  buf.addChar(0);
                  tok.stringVal = buf.getData();
                  state = -1;
                  }
               else {
                  *valueToChange = cur;
                  state = 6;
                  }
               break;
   
            case 9:                       // '-' sign: could be number or symbol
               if (isdigit(cur)) {
                  // when juxtaposed with a digit, the minus sign signals a
                  // negative numeric value, which is assumed to be an integer.
                  sign = -1;
                  state = 2;
                  tok.type = Token::integer;
                  tok.intVal = cur - '0';
                  }
               else {
                  state = -1;
                  putBack(cur);
                  }
               break;
   
            case 10:                    // floating point number - digit behind
                                        // d.p.
               if (isdigit(cur)) {
                  exp--;
                  tok.floatVal = tok.floatVal * 10  + cur - '0';
                  }
               else if (cur == 'e' || cur == 'E') {
                  tempExp = 0;
                  state = 11;
                  }
               else {
                  tok.floatVal *= sign;
                  while (exp < 0) {tok.floatVal /= 10; exp++;}
                  while (exp > 0) {tok.floatVal *= 10; exp--;}
                  state = -1;
                  putBack(cur);
                  }
               break;
    
            case 11:
               if (cur == '+') {
                  expSign = 1;
                  state = 12;
                  }
               else if (cur == '-') {
                  expSign = -1;
                  state = 12;
                  }
               else {
                  tok.floatVal *= sign;
                  while (exp < 0) {tok.floatVal /= 10; exp++;}
                  while (exp > 0) {tok.floatVal *= 10; exp--;}
                  state = -1;
                  putBack(cur);
                  }
               break;
   
            case 12:
               if (isdigit(cur)) {
                  tempExp = tempExp * 10 + cur - '0';
                  if (tempExp * expSign + exp > MAX_EXP)
                     tempExp = MAX_EXP;
                  else if (tempExp * expSign + exp < MIN_EXP)
                     tempExp = MIN_EXP;
                  }
               else {
                  exp += tempExp * expSign;
                  tok.floatVal *= sign;
                  while (exp < 0) {tok.floatVal /= 10; exp++;}
                  while (exp > 0) {tok.floatVal *= 10; exp--;}
                  state = -1;
                  putBack(cur);
                  }
               break;
   
               }
            
            }
   
         return tok;
         }
      
      int getLineNum()
         {
         return lineNum;
         }
         
      Tokenizer &setLineNum(int newLineNum)
         {
         lineNum = newLineNum;
         return *this;
         }
         
      const char *getSourceName()
         {
         return sourceName;
         }
         
      Tokenizer &setSourceName(const char *newName)
         {
         sourceName = newName;
         return *this;
         }
   }

